@@ -790,6 +790,9 @@ QRect ScreenRectangleForPoint(QPoint posGlobal)
 {
 #if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
 	const QScreen *screen = QGuiApplication::screenAt(posGlobal);
+	if (!screen) {
+		screen = QGuiApplication::primaryScreen();
+	}
 	return screen->availableGeometry();
 #else
 	const QDesktopWidget *desktop = QApplication::desktop();
@@ -840,6 +843,8 @@ void Window::SetPositionRelative(PRectangle rc, const Window *relativeTo)
 		ox = rectDesk.right() - sizex;
 	if (oy + sizey > rectDesk.bottom())
 		oy = rectDesk.bottom() - sizey;
+	if (oy < rectDesk.top())
+		oy = rectDesk.top();
 
 	Q_ASSERT(wid);
 	window(wid)->move(ox, oy);
