@@ -41,21 +41,25 @@ struct SelectedRange {
 	}
 };
 
+struct FilePosition {
+	SelectedRange selection;
+	SA::Line scrollPosition = 0;
+	FilePosition() = default;
+	FilePosition(SelectedRange selection_, SA::Line scrollPosition_) noexcept :
+		selection(selection_), scrollPosition(scrollPosition_) {
+	}
+};
+
 class RecentFile : public FilePath {
 public:
-	SelectedRange selection;
-	SA::Line scrollPosition;
-	RecentFile() {
-		scrollPosition = 0;
-	}
-	RecentFile(const FilePath &path_, SelectedRange selection_, SA::Line scrollPosition_) :
-		FilePath(path_), selection(selection_), scrollPosition(scrollPosition_) {
+	FilePosition filePosition;
+	RecentFile() = default;
+	RecentFile(const FilePath &path_, const FilePosition &filePosition_) :
+		FilePath(path_), filePosition(filePosition_) {
 	}
 	void Init() noexcept override {
 		FilePath::Init();
-		selection.position = SA::InvalidPosition;
-		selection.anchor = SA::InvalidPosition;
-		scrollPosition = 0;
+		filePosition = FilePosition();
 	}
 };
 
@@ -110,7 +114,7 @@ public:
 	void SetTimeFromFile();
 
 	void DocumentModified() noexcept;
-	bool NeedsSave(int delayBeforeSave) const;
+	bool NeedsSave(int delayBeforeSave) const noexcept;
 
 	void CompleteLoading() noexcept;
 	void CompleteStoring();
@@ -152,13 +156,13 @@ public:
 	BufferIndex size() const noexcept;
 	void Allocate(BufferIndex maxSize);
 	BufferIndex Add();
-	BufferIndex GetDocumentByWorker(const FileWorker *pFileWorker) const;
-	BufferIndex GetDocumentByName(const FilePath &filename, bool excludeCurrent=false);
+	BufferIndex GetDocumentByWorker(const FileWorker *pFileWorker) const noexcept;
+	BufferIndex GetDocumentByName(const FilePath &filename, bool excludeCurrent=false) const noexcept;
 	void RemoveInvisible(BufferIndex index);
 	void RemoveCurrent();
 	BufferIndex Current() const noexcept;
-	Buffer *CurrentBuffer();
-	const Buffer *CurrentBufferConst() const;
+	Buffer *CurrentBuffer() noexcept;
+	const Buffer *CurrentBufferConst() const noexcept;
 	void SetCurrent(BufferIndex index) noexcept;
 	BufferIndex StackNext();
 	BufferIndex StackPrev();
@@ -276,6 +280,20 @@ struct SystemAppearance {
 // Titles appear different in menus and tabs
 enum class Title { menu, tab };
 
+enum class GrepFlags {
+	none = 0,
+	wholeWord = 1,
+	matchCase = 2,
+	stdOut = 4,
+	dot = 8,
+	binary = 16,
+	scroll = 32
+};
+
+constexpr GrepFlags operator|(GrepFlags a, GrepFlags b) noexcept {
+	return static_cast<GrepFlags>(static_cast<int>(a) | static_cast<int>(b));
+}
+
 class SciTEBase : public ExtensionAPI, public Searcher, public WorkerListener {
 protected:
 	bool needIdle;
@@ -484,10 +502,10 @@ protected:
 	bool IsBufferAvailable() const noexcept;
 	bool CanMakeRoom(bool maySaveIfDirty = true);
 	void SetDocumentAt(BufferIndex index, bool updateStack = true);
-	Buffer *CurrentBuffer() {
+	Buffer *CurrentBuffer() noexcept {
 		return buffers.CurrentBuffer();
 	}
-	const Buffer *CurrentBufferConst() const {
+	const Buffer *CurrentBufferConst() const noexcept {
 		return buffers.CurrentBufferConst();
 	}
 	void SetBuffersMenu();
@@ -581,6 +599,7 @@ protected:
 	bool Open(const FilePath &file, OpenFlags of = ofNone);
 	bool OpenSelected();
 	void Revert();
+	std::string_view TextAsView();
 	FilePath SaveName(const char *ext) const;
 	enum SaveFlags {
 		sfNone = 0, 		// Default
@@ -810,8 +829,8 @@ protected:
 	bool AddFileToBuffer(const BufferState &bufferState);
 	void AddFileToStack(const RecentFile &file);
 	void RemoveFileFromStack(const FilePath &file);
-	RecentFile GetFilePosition();
-	void DisplayAround(const RecentFile &rf);
+	FilePosition GetFilePosition();
+	void DisplayAround(const FilePosition &rf);
 	void StackMenu(int pos);
 	void StackMenuNext();
 	void StackMenuPrev();
@@ -860,7 +879,9 @@ protected:
 	void SetOneStyle(GUI::ScintillaWindow &win, int style, const StyleDefinition &sd);
 	void SetStyleBlock(GUI::ScintillaWindow &win, const char *lang, int start, int last);
 	void SetStyleFor(GUI::ScintillaWindow &win, const char *lang);
-	static void SetOneIndicator(GUI::ScintillaWindow &win, int indicator, const IndicatorDefinition &ind);
+	static void SetOneIndicator(GUI::ScintillaWindow &win, SA::IndicatorNumbers indicator, const IndicatorDefinition &ind);
+	void SetIndicatorFromProperty(GUI::ScintillaWindow &win, SA::IndicatorNumbers indicator, const std::string &propertyName);
+	void SetMarkerFromProperty(GUI::ScintillaWindow &win, int marker, const std::string &propertyName);
 	void ReloadProperties();
 
 	void CheckReload();
@@ -891,14 +912,11 @@ protected:
 	virtual bool IsStdinBlocked();
 	void OpenFromStdin(bool UseOutputPane);
 	void OpenFilesFromStdin();
-	enum GrepFlags {
-		grepNone = 0, grepWholeWord = 1, grepMatchCase = 2, grepStdOut = 4,
-		grepDot = 8, grepBinary = 16, grepScroll = 32
-	};
 	virtual bool GrepIntoDirectory(const FilePath &directory);
-	void GrepRecursive(GrepFlags gf, const FilePath &baseDir, const char *searchString, const GUI::gui_char *fileTypes);
-	void InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const GUI::gui_char *fileTypes,
-			  const char *search, SA::Position &originalEnd);
+	void GrepRecursive(GrepFlags gf, const FilePath &baseDir, const char *searchString,
+		GUI::gui_string_view fileTypes, GUI::gui_string_view excludedTypes);
+	void InternalGrep(GrepFlags gf, const FilePath &directory, GUI::gui_string_view fileTypes, GUI::gui_string_view excludedTypes,
+			  std::string_view search, SA::Position &originalEnd);
 	void EnumProperties(const char *propkind);
 	void SendOneProperty(const char *kind, const char *key, const char *val);
 	void PropertyFromDirector(const char *arg);
@@ -949,6 +967,7 @@ public:
 	virtual void WorkerCommand(int cmd, Worker *pWorker);
 };
 
+const char *LineEndString(SA::EndOfLine eolMode) noexcept;
 int ControlIDOfCommand(unsigned long) noexcept;
 SA::Colour ColourOfProperty(const PropSetFile &props, const char *key, SA::Colour colourDefault);
 SA::ColourAlpha ColourAlphaOfProperty(const PropSetFile &props, const char *key, SA::ColourAlpha colourDefault);
