@@ -375,12 +375,31 @@ void SciTEBase::SetStyleFor(GUI::ScintillaWindow &win, const char *lang) {
 	SetStyleBlock(win, lang, 0, StyleMax);
 }
 
-void SciTEBase::SetOneIndicator(GUI::ScintillaWindow &win, int indicator, const IndicatorDefinition &ind) {
-	win.IndicSetStyle(indicator, ind.style);
-	win.IndicSetFore(indicator, ind.colour);
-	win.IndicSetAlpha(indicator, ind.fillAlpha);
-	win.IndicSetOutlineAlpha(indicator, ind.outlineAlpha);
-	win.IndicSetUnder(indicator, ind.under);
+void SciTEBase::SetOneIndicator(GUI::ScintillaWindow &win, SA::IndicatorNumbers indicator, const IndicatorDefinition &ind) {
+	const int indic = static_cast<int>(indicator);
+	win.IndicSetStyle(indic, ind.style);
+	win.IndicSetFore(indic, ind.colour);
+	win.IndicSetAlpha(indic, ind.fillAlpha);
+	win.IndicSetOutlineAlpha(indic, ind.outlineAlpha);
+	win.IndicSetUnder(indic, ind.under);
+}
+
+void SciTEBase::SetIndicatorFromProperty(GUI::ScintillaWindow &win, SA::IndicatorNumbers indicator, const std::string &propertyName) {
+	const std::string indicatorString = props.GetExpandedString(propertyName.c_str());
+	if (!indicatorString.empty()) {
+		IndicatorDefinition modifiedIndicator(indicatorString);
+		SetOneIndicator(win, indicator, modifiedIndicator);
+	}
+}
+
+void SciTEBase::SetMarkerFromProperty(GUI::ScintillaWindow &win, int marker, const std::string &propertyName) {
+	const std::string markerString = props.GetExpandedString(propertyName.c_str());
+	if (!markerString.empty()) {
+		MarkerDefinition markerValue(markerString);
+		win.MarkerDefine(marker, markerValue.style);
+		win.MarkerSetFore(marker, markerValue.colour);
+		win.MarkerSetBack(marker, markerValue.back);
+	}
 }
 
 std::string SciTEBase::ExtensionFileName() const {
@@ -459,10 +478,10 @@ std::string SciTEBase::FindLanguageProperty(const char *pattern, const char *def
  * A list of all the properties that should be forwarded to Scintilla lexers.
  */
 static const char *propertiesToForward[] = {
-	"fold.lpeg.by.indentation",
-	"lexer.lpeg.color.theme",
-	"lexer.lpeg.home",
-	"lexer.lpeg.script",
+	"fold.scintillua.by.indentation",
+	"fold.scintillua.line.groups",
+	"fold.scintillua.on.zero.sum.lines",
+	"fold.scintillua.compact",
 //++Autogenerated -- run ../scripts/RegenerateSource.py to regenerate
 //**\(\t"\*",\n\)
 	"asp.default.language",
@@ -1521,12 +1540,28 @@ void SciTEBase::ReadProperties() {
 			highlightCurrentWordIndicator.fillAlpha = alphaIndicator;
 			highlightCurrentWordIndicator.under = underIndicator;
 		}
-		SetOneIndicator(wEditor, indicatorHighlightCurrentWord, highlightCurrentWordIndicator);
-		SetOneIndicator(wOutput, indicatorHighlightCurrentWord, highlightCurrentWordIndicator);
+		const SA::IndicatorNumbers indicatorNumHCWI = static_cast<SA::IndicatorNumbers>(indicatorHighlightCurrentWord);
+		SetOneIndicator(wEditor, indicatorNumHCWI,	highlightCurrentWordIndicator);
+		SetOneIndicator(wOutput, indicatorNumHCWI, highlightCurrentWordIndicator);
 		currentWordHighlight.isOnlyWithSameStyle = props.GetInt("highlight.current.word.by.style", 0) == 1;
 		HighlightCurrentWord(true);
 	}
 
+	SetIndicatorFromProperty(wEditor, SA::IndicatorNumbers::HistoryRevertedToOriginInsertion, "indicator.reverted.to.origin.insertion");
+	SetIndicatorFromProperty(wEditor, SA::IndicatorNumbers::HistoryRevertedToOriginDeletion, "indicator.reverted.to.origin.deletion");
+	SetIndicatorFromProperty(wEditor, SA::IndicatorNumbers::HistorySavedInsertion, "indicator.saved.insertion");
+	SetIndicatorFromProperty(wEditor, SA::IndicatorNumbers::HistorySavedDeletion, "indicator.saved.deletion");
+	SetIndicatorFromProperty(wEditor, SA::IndicatorNumbers::HistoryModifiedInsertion, "indicator.modified.insertion");
+	SetIndicatorFromProperty(wEditor, SA::IndicatorNumbers::HistoryModifiedDeletion, "indicator.modified.deletion");
+	SetIndicatorFromProperty(wEditor, SA::IndicatorNumbers::HistoryRevertedToModifiedInsertion, "indicator.reverted.to.modified.insertion");
+	SetIndicatorFromProperty(wEditor, SA::IndicatorNumbers::HistoryRevertedToModifiedDeletion, "indicator.reverted.to.modified.deletion");
+
+	constexpr size_t markerHistory = static_cast<size_t>(SA::MarkerOutline::HistoryRevertedToOrigin);
+	SetMarkerFromProperty(wEditor, markerHistory+0, "marker.reverted.to.origin");
+	SetMarkerFromProperty(wEditor, markerHistory+1, "marker.saved");
+	SetMarkerFromProperty(wEditor, markerHistory+2, "marker.modified");
+	SetMarkerFromProperty(wEditor, markerHistory+3, "marker.reverted.to.modified");
+
 	ReadEditorConfig(fileNameForExtension);
 
 	if (extender) {
@@ -1631,6 +1666,50 @@ void SciTEBase::ReadEditorConfig(const std::string &fileNameForExtension) {
 	}
 }
 
+namespace {
+
+// Scintillua's style numbers are not constant, so ask it for names of styles and create a
+// mapping of style numbers to more constant style definitions.
+// For example, if Scintillua reports for the cpp lexer that style number 2 is a 'comment',
+// create the property:
+//   style.scintillua.cpp.2=$(scintillua.styles.comment)
+// That way the user can define 'scintillua.styles.comment' once and it will be used for whatever
+// the style number for comments is in any given lexer.
+// Similarly, if Scintillua reports for the lua lexer that style number 20 is 'string.longstring',
+// create the property:
+//   style.scintillua.lua.20=$(scintillua.styles.string),$(scintillua.styles.string.longstring)
+void SetScintilluaStyles(GUI::ScintillaWindow &wEditor, PropSetFile& props, const char *languageName) {
+	const auto setStyle = [&wEditor, &props, &languageName](int style) {
+		std::string finalPropStr;
+		const std::string &name = wEditor.NameOfStyle(style);
+		size_t end = std::string::npos;
+		do {
+			end = name.find('.', ++end);
+			char propStr[128] = "";
+			sprintf(propStr, "$(scintillua.styles.%s),", end == std::string::npos ?
+				name.c_str() : name.substr(0, end).c_str());
+			finalPropStr += propStr;
+		} while (end != std::string::npos);
+		char key[256] = "";
+		sprintf(key, "style.%s.%0d", languageName, style);
+		props.Set(key, finalPropStr.c_str());
+	};
+	const int namedStyles = wEditor.NamedStyles(); // this count includes predefined styles
+	constexpr int LastPredefined = static_cast<int>(Scintilla::StylesCommon::LastPredefined);
+	constexpr int numPredefined = LastPredefined - StyleDefault + 1;
+	for (int i = 0; i < std::min(namedStyles - numPredefined, StyleDefault); i++) {
+		setStyle(i);
+	}
+	for (int i = StyleDefault; i <= LastPredefined; i++) {
+		setStyle(i);
+	}
+	for (int i = LastPredefined + 1; i < namedStyles; i++) {
+		setStyle(i);
+	}
+}
+
+}
+
 void SciTEBase::ReadFontProperties() {
 	const std::string monospaceFonts = props.GetExpandedString("font.monospaced.list");
 	monospacedList = StringSplit(monospaceFonts, ';');
@@ -1638,14 +1717,8 @@ void SciTEBase::ReadFontProperties() {
 	char key[200] = "";
 	const char *languageName = language.c_str();
 
-	if (StartsWith(languageName, "lpeg.") && language.length() < 240) {
-		// Retrieve style info.
-		char propStr[256] = "";
-		for (int i = 0; i < StyleMax; i++) {
-			sprintf(key, "style.%s.%0d", languageName, i);
-			wEditor.PrivateLexerCall(i - StyleMax, propStr);
-			props.Set(key, propStr);
-		}
+	if (StartsWith(languageName, "scintillua.") && language.length() < 240) {
+		SetScintilluaStyles(wEditor, props, languageName);
 	}
 
 	// Set styles
