@@ -104,6 +104,49 @@ ViewStyle::ViewStyle(size_t stylesSize_) :
 	indicators[1] = Indicator(IndicatorStyle::TT, ColourRGBA(0, 0, 0xff));
 	indicators[2] = Indicator(IndicatorStyle::Plain, ColourRGBA(0xff, 0, 0));
 
+	// Reverted to origin
+	constexpr ColourRGBA revertedToOrigin(0x40, 0xA0, 0xBF);
+	// Saved
+	constexpr ColourRGBA saved(0x0, 0xA0, 0x0);
+	// Modified
+	constexpr ColourRGBA modified(0xFF, 0x80, 0x0);
+	// Reverted to change
+	constexpr ColourRGBA revertedToChange(0xA0, 0xC0, 0x0);
+
+	// Edition indicators
+	constexpr size_t indexHistory = static_cast<size_t>(IndicatorNumbers::HistoryRevertedToOriginInsertion);
+
+	indicators[indexHistory+0] = Indicator(IndicatorStyle::CompositionThick, revertedToOrigin, false, 30, 60);
+	indicators[indexHistory+1] = Indicator(IndicatorStyle::Point, revertedToOrigin);
+	indicators[indexHistory+2] = Indicator(IndicatorStyle::CompositionThick, saved, false, 30, 60);
+	indicators[indexHistory+3] = Indicator(IndicatorStyle::Point, saved);
+	indicators[indexHistory+4] = Indicator(IndicatorStyle::CompositionThick, modified, false, 30, 60);
+	indicators[indexHistory+5] = Indicator(IndicatorStyle::PointTop, modified);
+	indicators[indexHistory+6] = Indicator(IndicatorStyle::CompositionThick, revertedToChange, false, 30, 60);
+	indicators[indexHistory+7] = Indicator(IndicatorStyle::Point, revertedToChange);
+
+	// Edition markers
+	// Reverted to origin
+	constexpr size_t indexHistoryRevertedToOrigin = static_cast<size_t>(MarkerOutline::HistoryRevertedToOrigin);
+	markers[indexHistoryRevertedToOrigin].back = revertedToOrigin;
+	markers[indexHistoryRevertedToOrigin].fore = revertedToOrigin;
+	markers[indexHistoryRevertedToOrigin].markType = MarkerSymbol::Bar;
+	// Saved
+	constexpr size_t indexHistorySaved = static_cast<size_t>(MarkerOutline::HistorySaved);
+	markers[indexHistorySaved].back = saved;
+	markers[indexHistorySaved].fore = saved;
+	markers[indexHistorySaved].markType = MarkerSymbol::Bar;
+	// Modified
+	constexpr size_t indexHistoryModified = static_cast<size_t>(MarkerOutline::HistoryModified);
+	markers[indexHistoryModified].back = Platform::Chrome();
+	markers[indexHistoryModified].fore = modified;
+	markers[indexHistoryModified].markType = MarkerSymbol::Bar;
+	// Reverted to change
+	constexpr size_t indexHistoryRevertedToModified = static_cast<size_t>(MarkerOutline::HistoryRevertedToModified);
+	markers[indexHistoryRevertedToModified].back = revertedToChange;
+	markers[indexHistoryRevertedToModified].fore = revertedToChange;
+	markers[indexHistoryRevertedToModified].markType = MarkerSymbol::Bar;
+
 	technology = Technology::Default;
 	indicatorsDynamic = false;
 	indicatorsSetFore = false;
@@ -241,6 +284,7 @@ ViewStyle::ViewStyle(const ViewStyle &source) : ViewStyle(source.styles.size())
 	ms = source.ms;
 	maskInLine = source.maskInLine;
 	maskDrawInText = source.maskDrawInText;
+	maskDrawWrapped = source.maskDrawWrapped;
 	fixedColumnWidth = source.fixedColumnWidth;
 	marginInside = source.marginInside;
 	textStart = source.textStart;
@@ -304,6 +348,17 @@ void ViewStyle::CalculateMarginWidthAndMask() noexcept {
 			break;
 		}
 	}
+	maskDrawWrapped = 0;
+	for (int markBit = 0; markBit < 32; markBit++) {
+		const int maskBit = 1U << markBit;
+		switch (markers[markBit].markType) {
+		case MarkerSymbol::Bar:
+			maskDrawWrapped |= maskBit;
+			break;
+		default:	// Other marker types do not affect the masks
+			break;
+		}
+	}
 }
 
 void ViewStyle::Refresh(Surface &surface, int tabInChars) {
@@ -450,6 +505,9 @@ void ViewStyle::CalcLargestMarkerHeight() noexcept {
 			if (marker.image && marker.image->GetHeight() > largestMarkerHeight)
 				largestMarkerHeight = marker.image->GetHeight();
 			break;
+		case MarkerSymbol::Bar:
+			largestMarkerHeight = lineHeight + 2;
+			break;
 		default:	// Only images have their own natural heights
 			break;
 		}
@@ -711,10 +769,15 @@ FontRealised *ViewStyle::Find(const FontSpecification &fs) {
 }
 
 void ViewStyle::FindMaxAscentDescent() noexcept {
-	for (const auto &font : fonts) {
-		if (maxAscent < font.second->measurements.ascent)
-			maxAscent = font.second->measurements.ascent;
-		if (maxDescent < font.second->measurements.descent)
-			maxDescent = font.second->measurements.descent;
+	for (size_t i = 0; i < styles.size(); i++) {
+		if (i == StyleCallTip)
+			continue;
+
+		const auto &style = styles[i];
+
+		if (maxAscent < style.ascent)
+			maxAscent = style.ascent;
+		if (maxDescent < style.descent)
+			maxDescent = style.descent;
 	}
 }
